#define F_CPU	1000000UL

#include <avr/io.h>
#include <avr/iotn44.h>
#include <util/delay.h>

#define True	1
#define False	0

//>>>>>>>>>>>>>>>>>>>>>>>>>>>>

#define USI_MOSI	PINA6
#define USI_MISO	PINA5
#define USI_SCK	PINA4
#define USI_SS		PINA7
#define USI_DDR	DDRA
#define USI_PORT	PORTA

void USI_SPI_init(uint8_t mode){
	if(1 == mode){ // Master mode
		// configure USI pins: MOSI, SCK, SS as outputs
		USI_DDR |= (1<<USI_MOSI) | (1<<USI_SCK) | (1<<USI_SS);
		// set SS high
		USI_PORT |= (1<<USI_SS);
		// Configure MISO as an input & enable pull-up resistor
		USI_DDR &= ~(1<<USI_MISO);
		USI_PORT |= (1<<USI_MISO);
		// Clear OVF interrupt flag
		USISR |= (1<<USIOIF);
		USICR |= (1<<USIWM0) | (1<<USICS1) | (1<<USICLK) | (1<<USITC);
	}
	else if(mode == 0){ // Slave mode
		// Configure USI pins: MISO, SCK, SS as input & MOSI as output
		USI_DDR &= ~((1<<USI_MISO)|(1<<USI_SCK)|(1<<USI_SS));
		USI_DDR |= (1<<USI_MOSI);
		USI_PORT |= (1<<USI_MISO) | (1<<USI_SCK); // Set the internal Pull-up resistor
		// Configure 3-wire Slave mode, enable OVF interrupt
		USICR |= (1<<USIWM0) | (1<<USIOIE) | (1<<USICS1);
		USISR |= (1<<USIOIF);
	}
	else{
		DDRB |= (1<<PINB0);
		PORTB |= (1<<PINB0);
	}
}

void USI_SPI_transfer(uint8_t dataIn){
	// Set SS pin LOW
	USI_PORT &= ~(1<<USI_SS);

	do{
		// Load the data in USI data reg.
		USIDR = dataIn;
		// Toggle the SCK pin
		USICR |= (1<<USIWM0) | (1<<USICS1) | (1<<USICLK) | (1<<USITC);
	}while((USISR & (1<<USIOIF)) == 0);
	// Set back USI_SS pin HIGH
	USI_PORT |= (1<<USI_SS);
}

uint8_t USI_SPI_receiver(void){
	 USISR |= (1<<USIOIF);
	 while((USISR & (1<<USIOIF)) != 0);
	// Return USI buffer reg. content
	return USIBR;
}

//<<<<<<<<<<<<<<<<<<<<<<<<<<<<

void setColour(uint8_t value){
	 if(value == 1){ // White colour
		  PORTA &= ~((1<<PINA0)|(1<<PINA1)|(1<<PINA2));
	 }
	 else if(value == 2){	 //Blue
		  PORTA &= ~(1<<PINA0);
		  PORTA |= (1<<PINA1)|(1<<PINA2);
	 }
	 else if(value == 3){	 // Green
		  PORTA &= ~(1<<PINA1);
		  PORTA |= (1<<PINA0)|(1<<PINA2);
	 }
	 else if(value == 4){	 //Red
		  PORTA &= ~(1<<PINA2);
		  PORTA |= (1<<PINA1)|(1<<PINA0);
	 }
	 else if(value == 5){	 //yellow
		  PORTA &= ~((1<<PINA1)|(1<<PINA2));
		  PORTA |= (1<<PINA0);
	 }
	 else if(value == 6){	 //pink
		  PORTA &= ~((1<<PINA0)|(1<<PINA1));
		  PORTA |= (1<<PINA2);
	 }
	 else if(value == 7){	 //blue-ish
		  PORTA &= ~((1<<PINA0)|(1<<PINA2));
		  PORTA |= (1<<PINA1);
	 }
	 _delay_ms(1000);
}

int main(){
	 DDRA |= (1<<PINA0)|(1<<PINA1)|(1<<PINA2);
	 PORTA |= (1<<PINA0)|(1<<PINA1)|(1<<PINA2);

	 USI_SPI_init(0);

	 uint8_t dataIn = 0;

	 int i;
	 for (i=0; i<=7; i++){
		  setColour(i);
	 }
	 PORTA |= (1<<PINA0) | (1<<PINA1) | (1<<PINA2);

	 while(True){
		  dataIn = USI_SPI_receiver();
		  setColour(dataIn);
		  dataIn = 0;
	 }
}
